/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { TinyEmitter } from 'tiny-emitter';
import { ExportedChange } from './Exporter';
export declare enum Events {
    SheetAdded = "sheetAdded",
    SheetRemoved = "sheetRemoved",
    SheetRenamed = "sheetRenamed",
    NamedExpressionAdded = "namedExpressionAdded",
    NamedExpressionRemoved = "namedExpressionRemoved",
    ValuesUpdated = "valuesUpdated"
}
export interface Listeners {
    /**
     * The event is triggered upon adding a sheet anywhere inside the workbook.
     *
     * @event
     *
     * @param {string} addedSheetDisplayName the name of added sheet
     *
     * @category Sheet
     */
    sheetAdded: (addedSheetDisplayName: string) => any;
    /**
     * The event is triggered upon removing a sheet from anywhere inside the workbook.
     *
     * @event
     *
     * @param {string} removedSheetDisplayName the name of removed sheet
     * @param {ExportedChange[]} changes the values and location of applied changes
     *
     * @category Sheet
     */
    sheetRemoved: (removedSheetDisplayName: string, changes: ExportedChange[]) => any;
    /**
     * The event is triggered upon renaming a sheet anywhere inside the workbook.
     *
     * @event
     *
     * @param {string} oldDisplayName the old name of a sheet before renaming
     * @param {string} newDisplayName the new name of the sheet after renaming
     *
     * @category Sheet
     */
    sheetRenamed: (oldDisplayName: string, newDisplayName: string) => any;
    /**
     * The event is triggered upon adding a named expression with specified values and location.
     *
     * @event
     *
     * @param {string} namedExpressionName the name of added expression
     * @param {ExportedChange[]} changes the values and location of applied changes
     *
     * @category Named Expression
     */
    namedExpressionAdded: (namedExpressionName: string, changes: ExportedChange[]) => any;
    /**
     * The event is triggered upon removing a named expression with specified values and from an indicated location.
     *
     * @event
     *
     * @param {string} namedExpressionName the name of removed expression
     * @param {ExportedChange[]} changes the values and location of applied changes
     *
     * @category Named Expression
     */
    namedExpressionRemoved: (namedExpressionName: string, changes: ExportedChange[]) => any;
    /**
     * The event is triggered upon changing values in a specified location.
     *
     * @event
     *
     * @param {ExportedChange[]} changes the values and location of applied changes
     *
     * @category Values
     */
    valuesUpdated: (changes: ExportedChange[]) => any;
}
export interface TypedEmitter {
    on<Event extends keyof Listeners>(s: Event, listener: Listeners[Event]): void;
    off<Event extends keyof Listeners>(s: Event, listener: Listeners[Event]): void;
    once<Event extends keyof Listeners>(s: Event, listener: Listeners[Event]): void;
}
export declare class Emitter extends TinyEmitter implements TypedEmitter {
    emit<Event extends keyof Listeners>(event: Event, ...args: Parameters<Listeners[Event]>): this;
}
