/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { InternalScalarValue, SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
/**
 * Interpreter plugin containing date-specific functions
 */
export declare class DateTimePlugin extends FunctionPlugin {
    static implementedFunctions: {
        DATE: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        TIME: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        MONTH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        YEAR: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        HOUR: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        MINUTE: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        SECOND: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        TEXT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        EOMONTH: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
            })[];
        };
        DAY: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        DAYS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        WEEKDAY: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                minValue?: undefined;
            })[];
        };
        WEEKNUM: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                minValue?: undefined;
            })[];
        };
        ISOWEEKNUM: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        DATEVALUE: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        TIMEVALUE: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        NOW: {
            method: string;
            parameters: never[];
            isVolatile: boolean;
        };
        TODAY: {
            method: string;
            parameters: never[];
            isVolatile: boolean;
        };
        EDATE: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
            })[];
        };
        DAYS360: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: boolean;
                minValue?: undefined;
            })[];
        };
        DATEDIF: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
            })[];
        };
        YEARFRAC: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                minValue: number;
                maxValue: number;
            })[];
        };
        INTERVAL: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        NETWORKDAYS: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue?: undefined;
            })[];
        };
        'NETWORKDAYS.INTL': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                minValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue?: undefined;
                defaultValue?: undefined;
            })[];
        };
        WORKDAY: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue?: undefined;
            })[];
        };
        'WORKDAY.INTL': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                defaultValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                defaultValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                minValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue?: undefined;
                defaultValue?: undefined;
            })[];
        };
    };
    /**
     * Corresponds to DATE(year, month, day)
     *
     * Converts a provided year, month and day into date
     *
     * @param ast
     * @param formulaAddress
     */
    date(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    time(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    eomonth(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    day(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    days(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    /**
     * Corresponds to MONTH(date)
     *
     * Returns the month of the year specified by a given date
     *
     * @param ast
     * @param formulaAddress
     */
    month(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    /**
     * Corresponds to YEAR(date)
     *
     * Returns the year specified by a given date
     *
     * @param ast
     * @param formulaAddress
     */
    year(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    hour(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    minute(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    second(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    /**
     * Corresponds to TEXT(number, format)
     *
     * Tries to convert number to specified date format.
     *
     * @param ast
     * @param formulaAddress
     */
    text(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    weekday(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    weeknum(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    isoweeknum(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    private isoweeknumCore;
    datevalue(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    timevalue(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    now(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    today(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    edate(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    datedif(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    days360(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    private days360Core;
    yearfrac(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    interval(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    networkdays(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    networkdaysintl(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    workday(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    workdayintl(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    private networkdayscore;
    private workdaycore;
    private countWorkdays;
    private simpleRangeToFilteredHolidays;
}
